var request = require("request");

function VideoProxy() {
    this._successCallback = null;
    this._errorCallback = null;
    this._readyForFrame = true;
}

/**
 * Starts to read the stream the "success" callback is called on every frame
 * @param success(base64)
 * @param error(error)
 * @param url
 * @param user
 * @param password
 */
VideoProxy.prototype.startStream = function startStream(success, error, url, user, password) {

    var config = {
        timeout: 6000
    };
    if (user && password) {
        config.auth = {
            user: user,
            pass: password,
            sendImmediately: false // Allows digest Authentication
        };
    }

    this._successCallback = function(b64String) {
        if (this._readyForFrame) {
            this._readyForFrame = false;
            success(b64String);
        }
    };
    this._errorCallback = function(err) {
        this.stopStream();
        error(err);
    };

    // Validate the contentType to prevent us from downloading a MJPG stream instead of JPG
    var contentTypeRequest = request(url, config).on("error", this._errorCallback.bind(this)).on("response", function(response) {
        contentTypeRequest.abort();
        if(response.statusCode !== 200) {
            this._errorCallback(response.statusCode);
            return;
        }
        // Note: Its important to stop any currently active streams. This plugin can only handle one stream at a time!
        this.stopStream();
        if (response.headers['content-type'].indexOf("multipart/x-mixed-replace") !== -1) {
            this._fetchMjpgFrame(url, config);
        } else {
            this._fetchJpg(url, config);
        }
    }.bind(this))
};

/**
 * Reads the MJPG stream and extracts every frame until VideoProxy.stopStream() is called
 * @param url
 * @param config
 * @private
 */
VideoProxy.prototype._fetchMjpgFrame = function _fetchMjpgFrame(url, config) {
    var chunks = [],
        soi = new Buffer.from([0xff, 0xd8]), // Start Of Input
        eoi = new Buffer.from([0xff, 0xd9]); // End Of Input

    this._request = request(url, config).on("error", function(error) {
        this._errorCallback(error);
    }.bind(this)).on("response", function(response) {
        if(response.statusCode !== 200) {
            this._request.abort();
            this._errorCallback(response.statusCode);
        }

        this._request.on('data',function (data) {
            if(data.indexOf(soi) !== -1) { // Check if the data chunk contains the JPEG start magic
                chunks = [];
                chunks.push(data.slice(data.indexOf(soi), data.length));
            } else {
                if(data.indexOf(eoi) === -1) {  // The data chunk doesn't contain the JPEG end magic, add it to the the received chunks
                    chunks.push(data);
                } else { // The data chunk contains the JPEG end magic, extract the data until  the JPEG end magic and send the buffer to the successCallback
                    chunks.push(data.slice(0, data.indexOf(eoi) + 1 + 1)); // +1 is for the second bit of the eoi buffer, the other +1 is for get the length instead of the index
                    this._successCallback(Buffer.concat(chunks).toString("base64"));
                }
            }
        }.bind(this));
    }.bind(this));
};

/**
 * Fetches JPG images until VideoProxy.stopStream() is called
 * @param url
 * @param config
 * @private
 */
VideoProxy.prototype._fetchJpg = function _fetchJpg(url, config) {
    var callingArgs = arguments,
        dataChuncks = [];

    this._request = request(url, config).on("error", function(error) {
        this._errorCallback(error);
    }.bind(this)).on("response", function(response) {
        if (response.statusCode !== 200) {
            this._request.abort();
            this._errorCallback(response.statusCode);
            return;
        }

        this._request.on('data', function(chunk) {
            dataChuncks.push(chunk); // chunk is one symbol
        }.bind(this)).on("end", function() {
            this._fetchJpg.apply(this, callingArgs);
            this._successCallback(Buffer.concat(dataChuncks).toString('base64'));
        }.bind(this));
    }.bind(this));
};

/**
 * Stops the currently active stream
 */
VideoProxy.prototype.stopStream = function stopStream() {
    this._request && this._request.abort();
    this.imageProcessed();
};

/**
 * Call this when the UI is ready processing the image,
 * no further frame will be emitted to the callback until this method is called
 */
VideoProxy.prototype.imageProcessed = function imageProcessed() {
    this._readyForFrame = true;
};

module.exports = VideoProxy;
